==> julynter-0.3.3-py3-none-any.whl: package/lib/linterlab/notebookhandler.js <==
import { Signal } from '@lumino/signaling';
import { Languages } from '../linter/languages';
import { Linter } from '../linter/lint';
import { GroupGenerator, ItemGenerator } from './itemgenerator';
import { OptionsManager } from './optionsmanager';
export class NotebookHandler {
    constructor(docManager, session, nbPanel, config, em, eh, update) {
        this._kernelRestarted = new Signal(this);
        this._disposed = new Signal(this);
        this._inspected = new Signal(this);
        this._isDisposed = false;
        this._eh = eh;
        try {
            this._docManager = docManager;
            this._session = session;
            this._nbPanel = nbPanel;
            this._experimentManager = em;
            this._update = update;
            this._panelId = this._nbPanel.id;
            this._language = Languages.GENERIC;
            this.options = new OptionsManager(nbPanel, config, em, eh, update);
            this._attempts = 0;
            this.update = {};
            this.hasKernel = false;
            this._reportedStart = false;
            em.reportActivity(this, 'open');
            session.statusChanged.connect((sender, status) => {
                try {
                    const kdn = session.kernelDisplayName;
                    if (status.endsWith('restarting')) {
                        em.reportKernelActivity(this, 'restartKernel', kdn);
                        this._reportedStart = true;
                        this._kernelRestarted.emit(this._session.ready);
                    }
                    else if (status === 'unknown') {
                        em.reportKernelActivity(this, 'stopKernel', kdn);
                        this._reportedStart = false;
                    }
                    else if ((status === 'idle' || status === 'busy') &&
                        !this._reportedStart) {
                        em.reportKernelActivity(this, 'useKernel', kdn);
                        this._reportedStart = true;
                    }
                }
                catch (error) {
                    throw this._eh.report(error, 'NotebookHandler:constructor.statusChanged', [nbPanel.title.label, status]);
                }
            });
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:constructor', [
                nbPanel.title.label,
            ]);
        }
    }
    getKernelLanguage() {
        try {
            return this._session.session.kernel.info.then((infoReply) => {
                try {
                    const model = Languages.getScript(infoReply.language_info.name);
                    this._experimentManager.reportNotebookKernel(this, this._session.session.kernel.name, infoReply.language_info.name);
                    return model;
                }
                catch (error) {
                    throw this._eh.report(error, 'NotebookHandler:getKernelLanguage.then', [this._session.session.kernel.name, infoReply.language_info.name]);
                }
            });
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:getKernelLanguage', []);
        }
    }
    configureHandler(language) {
        try {
            this._language = language;
            this._ready = this._session.ready.then(() => {
                this._initOnKernel().then(() => {
                    this._session.iopubMessage.connect(this._queryCall.bind(this));
                    return;
                });
            });
            this._kernelRestarted.connect((sender, kernelReady) => {
                this._inspected.emit({
                    status: 'Restarting Kernel...',
                });
                // Emit restarting
                this._ready = kernelReady.then(() => {
                    this._initOnKernel().then(() => {
                        this._session.iopubMessage.connect(this._queryCall.bind(this));
                        this.performQuery();
                    });
                });
            });
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:configureHandler', [
                language.name,
            ]);
        }
    }
    disconnectHandler() {
        try {
            this.inspected.disconnect(this.onQueryUpdate, this);
            this.disposed.disconnect(this.onSourceDisposed, this);
            this._experimentManager.reportActivity(this, 'MoveOut');
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:disconnectHandler', []);
        }
    }
    connectHandler() {
        try {
            this.inspected.connect(this.onQueryUpdate, this);
            this.disposed.connect(this.onSourceDisposed, this);
            this.performQuery();
            this._experimentManager.reportActivity(this, 'MoveIn');
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:connectHandler', []);
        }
    }
    get name() {
        return this._session.path;
    }
    get id() {
        return this._panelId;
    }
    get nbPanel() {
        return this._nbPanel;
    }
    get experimentManager() {
        return this._experimentManager;
    }
    /**
     * A signal emitted when the handler is disposed.
     */
    get disposed() {
        return this._disposed;
    }
    get isDisposed() {
        return this._isDisposed;
    }
    get ready() {
        return this._ready;
    }
    /**
     * A signal emitted when an inspector value is generated.
     */
    get inspected() {
        return this._inspected;
    }
    /**
     * Disposes the kernel connector.
     */
    dispose() {
        try {
            if (this.isDisposed) {
                return;
            }
            this._experimentManager.reportCloseNotebook(this);
            this._isDisposed = true;
            this._disposed.emit(void 0);
            Signal.clearData(this);
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:dispose', []);
        }
    }
    /**
     * Lint notebook
     */
    lint() {
        try {
            const groupGenerator = new GroupGenerator(this._nbPanel, this._update, this._eh);
            const itemGenerator = new ItemGenerator(this._docManager, this, this._eh);
            const notebookMetadata = {
                title: this.nbPanel.title.label,
                cells: this.nbPanel.content
                    .widgets,
            };
            const linter = new Linter(this.options, this.update, this.hasKernel);
            const results = linter.generate(notebookMetadata, itemGenerator, groupGenerator);
            this._experimentManager.reportLinting(this, results);
            return results;
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:lint', []);
        }
    }
    _createPromise(error = null) {
        return new Promise((resolve, reject) => {
            if (!error) {
                resolve();
            }
            else {
                reject(error);
            }
        });
    }
    executeLanguageCommand(command, args, response) {
        try {
            if (this._language === null) {
                return this._createPromise('Language not loaded');
            }
            if (this._language[command] === null) {
                return this._createPromise();
            }
            const codefn = this._language[command];
            let code;
            if (codefn instanceof Function) {
                code = codefn(...args);
            }
            else {
                code = codefn;
            }
            if (code === null) {
                return this._createPromise();
            }
            const content = {
                code: code,
                stop_on_error: false,
                store_history: false,
            };
            return this.sendToKernel(content, response);
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:executeLanguageCommand', [
                command,
            ]);
        }
    }
    /**
     * Send a query command to the kernel
     */
    performQuery() {
        this.executeLanguageCommand('queryCommand', [this.options.checkRequirements()], this._handleQueryResponse.bind(this));
    }
    /**
     * Send message to kernel add a module
     */
    addModule(module) {
        this.executeLanguageCommand('addModuleCommand', [module, this.options.checkRequirements()], this._handleQueryResponse.bind(this));
    }
    /**
     * Initializes the kernel by running the set up script located at _initScriptPath.
     */
    _initOnKernel() {
        return this.executeLanguageCommand('initScript', [], null);
    }
    /*
     * Handle query response
     */
    _handleQueryResponse(msg, response) {
        try {
            const msgType = response.header.msg_type;
            let payload;
            let payloadError;
            let content;
            switch (msgType) {
                case 'execute_result':
                case 'display_data':
                    payload = response.content;
                    content = payload.data['text/plain'];
                    if (!this._language.julynterCode(content)) {
                        // A bug in Jupyter makes it return the wrong cell
                        break;
                    }
                    if (content.slice(0, 1) === "'" || content.slice(0, 1) === '"') {
                        content = content.slice(1, -1);
                        content = content.replace(/\\"/g, '"').replace(/\\'/g, "'");
                    }
                    this._inspected.emit({
                        status: '',
                        kernelName: this._session.kernelDisplayName || '',
                        result: JSON.parse(content),
                    });
                    break;
                case 'error':
                    payloadError = response.content;
                    if (payloadError.evalue.includes('julynter')) {
                        this._attempts += 1;
                        this._eh.report(`Failed to initialize scripts. Retrying ${this._attempts}/3\n${payloadError}`, 'NotebookHandler:_handleQueryResponse', [msg.code, response.content]);
                        if (this._attempts <= 3) {
                            this._inspected.emit({
                                status: 'Retrying to init',
                            });
                            this._initOnKernel().then(() => {
                                this._session.iopubMessage.connect(this._queryCall.bind(this));
                                this.performQuery();
                            });
                        }
                        else {
                            this._eh.report('Failed to initialize scripts after 3 attempts', 'NotebookHandler:_handleQueryResponse', [msg.code, response.content]);
                        }
                    }
                    break;
                default:
                    break;
            }
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:_handleQueryResponse', [
                msg.code,
                response.content,
            ]);
        }
    }
    /*
     * Invokes a inspection if the signal emitted from specified session is an 'execute_input' msg.
     */
    _queryCall(sess, args) {
        try {
            const msg = args;
            const msgType = msg.header.msg_type;
            let code;
            switch (msgType) {
                case 'execute_input':
                    code = msg.content.code;
                    if (!this._language.julynterCode(code)) {
                        this.performQuery();
                    }
                    break;
                default:
                    break;
            }
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:_queryCall', [
                args.content,
            ]);
        }
    }
    /**
     * Handle kernel signals.
     */
    onQueryUpdate(sender, update) {
        if (update.status !== '') {
            this.update = {};
            this.hasKernel = false;
        }
        else {
            this.hasKernel = true;
            this.update = update.result;
        }
    }
    /**
     * Handle disposed signals.
     */
    onSourceDisposed(sender, args) {
        return;
    }
    /**
     * Executes the given request on the kernel associated with the connector.
     */
    sendToKernel(content, ioCallback) {
        try {
            const kernel = this._session.session.kernel;
            if (!kernel) {
                return Promise.reject(new Error('Require kernel to perform advanced julynter operations!'));
            }
            const future = kernel.requestExecute(content, false);
            future.onIOPub = (msg) => {
                if (ioCallback !== null) {
                    ioCallback(content, msg);
                }
            };
            return future.done.then(() => {
                return;
            });
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:sendToKernel', [content]);
        }
    }
}

==> julynter-0.3.5-py3-none-any.whl: package/lib/linterlab/notebookhandler.js <==
import { Signal } from '@lumino/signaling';
import { Languages } from '../linter/languages';
import { Linter } from '../linter/lint';
import { GroupGenerator, ItemGenerator } from './itemgenerator';
import { OptionsManager } from './optionsmanager';
export class NotebookHandler {
    constructor(docManager, session, nbPanel, config, em, eh, update) {
        this._kernelRestarted = new Signal(this);
        this._disposed = new Signal(this);
        this._inspected = new Signal(this);
        this._isDisposed = false;
        this._eh = eh;
        try {
            this._docManager = docManager;
            this._session = session;
            this._nbPanel = nbPanel;
            this._experimentManager = em;
            this._update = update;
            this._panelId = this._nbPanel.id;
            this._language = Languages.GENERIC;
            this.options = new OptionsManager(nbPanel, config, em, eh, update);
            this.update = {};
            this.hasKernel = false;
            this._reportedStart = false;
            this._icomm = null;
            this._boundQueryCall = this._queryCall.bind(this);
            em.reportActivity(this, 'open');
            session.statusChanged.connect((sender, status) => {
                try {
                    const kdn = session.kernelDisplayName;
                    if (status.endsWith('restarting')) {
                        em.reportKernelActivity(this, 'restartKernel', kdn);
                        this._reportedStart = true;
                        this._kernelRestarted.emit(this._session.ready);
                    }
                    else if (status === 'unknown') {
                        em.reportKernelActivity(this, 'stopKernel', kdn);
                        this._reportedStart = false;
                    }
                    else if ((status === 'idle' || status === 'busy') &&
                        !this._reportedStart) {
                        em.reportKernelActivity(this, 'useKernel', kdn);
                        this._reportedStart = true;
                    }
                }
                catch (error) {
                    throw this._eh.report(error, 'NotebookHandler:constructor.statusChanged', [nbPanel.title.label, status]);
                }
            });
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:constructor', [
                nbPanel.title.label,
            ]);
        }
    }
    getKernelLanguage() {
        try {
            return this._session.session.kernel.info.then((infoReply) => {
                try {
                    const model = Languages.getScript(infoReply.language_info.name);
                    this._experimentManager.reportNotebookKernel(this, this._session.session.kernel.name, infoReply.language_info.name);
                    return model;
                }
                catch (error) {
                    throw this._eh.report(error, 'NotebookHandler:getKernelLanguage.then', [this._session.session.kernel.name, infoReply.language_info.name]);
                }
            });
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:getKernelLanguage', []);
        }
    }
    createComm() {
        const kernel = this._session.session.kernel;
        if (kernel) {
            kernel.registerCommTarget('julynter.comm', (comm, msg) => {
                this._icomm = comm;
                this._icomm.onMsg = this._receiveJulynterQuery.bind(this);
                // console.log('ICOMM!', this._icomm.commId);
            });
        }
    }
    configureHandler(language) {
        try {
            this._language = language;
            this._ready = this._session.ready.then(() => {
                this.createComm();
                this._initOnKernel();
            });
            this._kernelRestarted.connect((sender, kernelReady) => {
                this._inspected.emit({
                    status: 'Restarting Kernel...',
                });
                // Emit restarting
                this._ready = kernelReady.then(() => {
                    this.createComm();
                    this._initOnKernel();
                });
            });
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:configureHandler', [
                language.name,
            ]);
        }
    }
    disconnectHandler() {
        try {
            this.inspected.disconnect(this.onQueryUpdate, this);
            this.disposed.disconnect(this.onSourceDisposed, this);
            this._experimentManager.reportActivity(this, 'MoveOut');
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:disconnectHandler', []);
        }
    }
    connectHandler() {
        try {
            this.inspected.connect(this.onQueryUpdate, this);
            this.disposed.connect(this.onSourceDisposed, this);
            this.performQuery();
            this._experimentManager.reportActivity(this, 'MoveIn');
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:connectHandler', []);
        }
    }
    get name() {
        return this._session.path;
    }
    get id() {
        return this._panelId;
    }
    get nbPanel() {
        return this._nbPanel;
    }
    get experimentManager() {
        return this._experimentManager;
    }
    /**
     * A signal emitted when the handler is disposed.
     */
    get disposed() {
        return this._disposed;
    }
    get isDisposed() {
        return this._isDisposed;
    }
    get ready() {
        return this._ready;
    }
    /**
     * A signal emitted when an inspector value is generated.
     */
    get inspected() {
        return this._inspected;
    }
    /**
     * Disposes the kernel connector.
     */
    dispose() {
        try {
            if (this.isDisposed) {
                return;
            }
            this._experimentManager.reportCloseNotebook(this);
            this._isDisposed = true;
            this._disposed.emit(void 0);
            Signal.clearData(this);
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:dispose', []);
        }
    }
    /**
     * Lint notebook
     */
    lint() {
        try {
            const groupGenerator = new GroupGenerator(this._nbPanel, this._update, this._eh);
            const itemGenerator = new ItemGenerator(this._docManager, this, this._eh);
            const notebookMetadata = {
                title: this.nbPanel.title.label,
                cells: this.nbPanel.content
                    .widgets,
            };
            const linter = new Linter(this.options, this.update, this.hasKernel);
            const results = linter.generate(notebookMetadata, itemGenerator, groupGenerator);
            this._experimentManager.reportLinting(this, results);
            return results;
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:lint', []);
        }
    }
    _createPromise(error = null) {
        return new Promise((resolve, reject) => {
            if (!error) {
                resolve();
            }
            else {
                reject(error);
            }
        });
    }
    initScript() {
        try {
            if (this._language === null) {
                return this._createPromise('Language not loaded');
            }
            const code = this._language.initScript;
            if (code === null) {
                return this._createPromise();
            }
            const content = {
                code: code,
                stop_on_error: false,
                store_history: false,
                silent: true,
            };
            const kernel = this._session.session.kernel;
            if (!kernel) {
                return Promise.reject(new Error('Require kernel to perform advanced julynter operations!'));
            }
            const future = kernel.requestExecute(content, false);
            future.onIOPub = (msg) => {
                this.performQuery();
            };
            return future.done.then(() => {
                return;
            });
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:initScript', []);
        }
    }
    send(data) {
        const session = this._session.session;
        if (this._icomm &&
            session &&
            session.kernel &&
            session.kernel.hasComm(this._icomm.commId)) {
            this._icomm.send(data);
        }
    }
    /**
     * Send a query command to the kernel
     */
    performQuery() {
        this.send({
            operation: 'query',
            requirements: this.options.checkRequirements(),
        });
    }
    /**
     * Send message to kernel add a module
     */
    addModule(module) {
        this.send({
            operation: 'addModule',
            module: module,
            requirements: this.options.checkRequirements(),
        });
    }
    /**
     * Initializes the kernel by running the set up script located at _initScriptPath.
     */
    _initOnKernel() {
        return this.initScript().then(() => {
            this._session.iopubMessage.disconnect(this._boundQueryCall);
            this._session.iopubMessage.connect(this._boundQueryCall);
        });
    }
    /*
     * Handle query response
     */
    _receiveJulynterQuery(msg) {
        try {
            const operation = msg.content.data.operation;
            if (operation === 'queryResult') {
                this._inspected.emit({
                    status: '',
                    kernelName: this._session.kernelDisplayName || '',
                    result: msg.content.data,
                });
            }
            else if (operation === 'error') {
                this._eh.report('Failed to run ICOMM command', 'NotebookHandler:_receiveJulynterQuery', [msg]);
            }
            else if (operation === 'init') {
                this.performQuery();
            }
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:_receiveJulynterQuery', [
                msg,
            ]);
        }
    }
    /*
     * Invokes a inspection if the signal emitted from specified session is an 'execute_input' msg.
     */
    _queryCall(sess, args) {
        try {
            const msg = args;
            const msgType = msg.header.msg_type;
            switch (msgType) {
                case 'execute_input':
                    this.performQuery();
                    break;
                default:
                    break;
            }
        }
        catch (error) {
            throw this._eh.report(error, 'NotebookHandler:_queryCall', [
                args.content,
            ]);
        }
    }
    /**
     * Handle kernel signals.
     */
    onQueryUpdate(sender, update) {
        if (update.status !== '') {
            this.update = {};
            this.hasKernel = false;
        }
        else {
            this.hasKernel = true;
            this.update = update.result;
        }
    }
    /**
     * Handle disposed signals.
     */
    onSourceDisposed(sender, args) {
        return;
    }
}
